@@ -57,7 +57,8 @@ public class ColumnMetadataFilter extends BasicFilter {
         };
     }
 
-    public ProcessingStep createStep( Iterator argIt ) throws ArgException {
+    public ProcessingStep createStep( Iterator<String> argIt )
+            throws ArgException {
         String colidList = null;
         String rename = null;
         String units = null;
@@ -67,35 +68,35 @@ public class ColumnMetadataFilter extends BasicFilter {
         int[] shape = null;
         int elsize = -1;
         while ( argIt.hasNext() && colidList == null ) {
-            String arg = (String) argIt.next();
+            String arg = argIt.next();
             if ( arg.equals( "-name" ) && argIt.hasNext() ) {
                 argIt.remove();
-                rename = (String) argIt.next();
+                rename = argIt.next();
                 argIt.remove();
             }
             else if ( arg.startsWith( "-unit" ) && argIt.hasNext() ) {
                 argIt.remove();
-                units = (String) argIt.next();
+                units = argIt.next();
                 argIt.remove();
             }
             else if ( arg.equals( "-ucd" ) && argIt.hasNext() ) {
                 argIt.remove();
-                ucd = (String) argIt.next();
+                ucd = argIt.next();
                 argIt.remove();
             }
             else if ( arg.equals( "-utype" ) && argIt.hasNext() ) {
                 argIt.remove();
-                utype = (String) argIt.next();
+                utype = argIt.next();
                 argIt.remove();
             }
             else if ( arg.equals( "-desc" ) && argIt.hasNext() ) {
                 argIt.remove();
-                desc = (String) argIt.next();
+                desc = argIt.next();
                 argIt.remove();
             }
             else if ( arg.equals( "-shape" ) && argIt.hasNext() ) {
                 argIt.remove();
-                String shapeTxt = (String) argIt.next();
+                String shapeTxt = argIt.next();
                 argIt.remove();
                 try {
                     shape = DefaultValueInfo.unformatShape( shapeTxt );
@@ -107,7 +108,7 @@ public class ColumnMetadataFilter extends BasicFilter {
             }
             else if ( arg.equals( "-elsize" ) && argIt.hasNext() ) {
                 argIt.remove();
-                String elsizeTxt = (String) argIt.next();
+                String elsizeTxt = argIt.next();
                 argIt.remove();
                 try {
                     elsize = Integer.parseInt( elsizeTxt );
