@@ -31,7 +31,7 @@ public class SequentialViewFilter extends BasicFilter
         };
     }
 
-    public ProcessingStep createStep( Iterator argIt ) {
+    public ProcessingStep createStep( Iterator<String> argIt ) {
         return this;
     }
 
