@@ -5,7 +5,9 @@ import gnu.jel.DVMap;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.Iterator;
+import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Set;
 
 /**
  * An object which is able to read cell values by column name or number.
@@ -64,7 +66,8 @@ import java.util.List;
 public abstract class JELRowReader extends DVMap {
 
     private final Object[] args_;
-    private List constantList_;
+    private final List<NamedConstant> constantList_;
+    private final Set<Integer> translatedIcols_;
     private boolean isNullExpression_;
     private boolean failOnNull_;
 
@@ -95,7 +98,7 @@ public abstract class JELRowReader extends DVMap {
     /** Constant which effectively returns a null primitive. */
     private final Constant NULL_EXPRESSION_CONST = new Constant() {
         private final Byte b0 = new Byte( (byte) 0 );
-        public Class getContentClass() {
+        public Class<?> getContentClass() {
             return Byte.class;
         }
         public Object getValue() {
@@ -109,7 +112,8 @@ public abstract class JELRowReader extends DVMap {
      */
     public JELRowReader() {
         args_ = new Object[] { this };
-        constantList_ = new ArrayList();
+        constantList_ = new ArrayList<NamedConstant>();
+        translatedIcols_ = new LinkedHashSet<Integer>();
     }
 
     /**
@@ -160,11 +164,12 @@ public abstract class JELRowReader extends DVMap {
 
     /**
      * Returns the class of values returned by a given column.
+     * If no column with the given index exists, null should be returned.
      *
-     * @param  icol   column index
-     * @return  value class
+     * @param  icol   non-negative column index
+     * @return  value class, or null for non-existent column
      */
-    protected abstract Class getColumnClass( int icol );
+    protected abstract Class<?> getColumnClass( int icol );
 
     /**
      * Returns a boolean value for a cell of the current row.
@@ -299,8 +304,7 @@ public abstract class JELRowReader extends DVMap {
             return NULL_EXPRESSION_CONST;
         }
         else if ( getNullConstantIndex( name ) >= 0 ) {
-            Object value = ((NamedConstant)
-                            constantList_.get( getNullConstantIndex( name ) ))
+            Object value = constantList_.get( getNullConstantIndex( name ) )
                           .getValue();
             return value == null ? TRUE_CONST
                                  : FALSE_CONST;
@@ -366,7 +370,7 @@ public abstract class JELRowReader extends DVMap {
         /* See if it's a known special, and treat it specially if so. */
         int ispecial = getSpecialIndex( name );
         if ( ispecial >= 0 ) {
-            return getTypeName( ((NamedConstant) constantList_.get( ispecial ))
+            return getTypeName( constantList_.get( ispecial )
                                .getContentClass() );
         }
 
@@ -380,7 +384,10 @@ public abstract class JELRowReader extends DVMap {
         /* See if it's a known column, and get the return value type if so. */
         int icol = getColumnIndex( name );
         if ( icol >= 0 ) {
-            return getTypeName( getColumnClass( icol ) );
+            Class<?> clazz = getColumnClass( icol ); 
+            if ( clazz != null ) {
+                return getTypeName( clazz );
+            }
         }
 
         /* See if it's an object-valued column indicator, and return an Object
@@ -400,8 +407,7 @@ public abstract class JELRowReader extends DVMap {
         /* See if it's a known constant, and treat it specially if so. */
         int iconst = getConstantIndex( name );
         if ( iconst >= 0 ) {
-            return getTypeName( ((NamedConstant) constantList_.get( iconst ))
-                               .getContentClass() );
+            return getTypeName( constantList_.get( iconst ).getContentClass() );
         }
 
         /* If we haven't got it yet, we don't know what it is. */
@@ -442,18 +448,21 @@ public abstract class JELRowReader extends DVMap {
         /* See if it corresponds to a null column indicator. */
         int inul = getNullColumnIndex( name );
         if ( inul >= 0 ) {
+            translatedIcols_.add( new Integer( inul ) );
             return new Long( inul );
         }
 
         /* See if it corresponds to a column. */
         int icol = getColumnIndex( name );
         if ( icol >= 0 ) {
+            translatedIcols_.add( new Integer( icol ) );
             return new Integer( icol );
         }
 
         /* See if it corresponds to an object-valued column. */
         int iobj = getObjectColumnIndex( name );
         if ( iobj >= 0 ) {
+            translatedIcols_.add( new Integer( iobj ) );
             return new Integer( iobj );
         }
 
@@ -468,6 +477,27 @@ public abstract class JELRowReader extends DVMap {
         return null;
     }
 
+    /**
+     * Returns a set (no duplicated elements) of the column indices
+     * for which this RowReader has been asked to provide translation values.
+     * In practice that means the index of every table column which has
+     * been directly referenced in a JEL expression which this RowReader
+     * has been used to compile.
+     *
+     * @return  list of distinct column indices which this row reader
+     *          has had to reference in compiling JEL expressions
+     */
+    public int[] getTranslatedColumns() {
+        int ncol = translatedIcols_.size();
+        int[] icols = new int[ ncol ];
+        int i = 0;
+        for ( Integer ic : translatedIcols_ ) {
+            icols[ i++ ] = ic.intValue();
+        }
+        assert i == ncol;
+        return icols;
+    }
+
     /**
      * Returns the column index in the table model which corresponds to 
      * a given name.  The current formats are
@@ -579,7 +609,7 @@ public abstract class JELRowReader extends DVMap {
          * this name that has been encountered before.
          * If one is found return the index. */
         for ( int i = 0; i < constantList_.size(); i++ ) {
-            String cname = ((NamedConstant) constantList_.get( i )).getName();
+            String cname = constantList_.get( i ).getName();
             if ( cname.equals( name ) ) {
                 return i;
             }
@@ -651,7 +681,7 @@ public abstract class JELRowReader extends DVMap {
      * @return  value as an Object
      */
     private Object getConstantValue( int iconst ) {
-        return ((NamedConstant) constantList_.get( iconst )).getValue();
+        return constantList_.get( iconst ).getValue();
     }
 
     /**
@@ -867,7 +897,7 @@ public abstract class JELRowReader extends DVMap {
      * @return  the corresponding method name fragment
      * @see   "JEL manual"
      */
-    private static String getTypeName( Class clazz ) {
+    private static String getTypeName( Class<?> clazz ) {
         if ( clazz.equals( Boolean.class ) ) {
             return "Boolean";
         }
@@ -984,7 +1014,7 @@ public abstract class JELRowReader extends DVMap {
             return name_;
         }
 
-        public Class getContentClass() {
+        public Class<?> getContentClass() {
             return konst_.getContentClass();
         }
 
