@@ -36,6 +36,7 @@ import uk.ac.starlink.ttools.func.Strings;
 import uk.ac.starlink.ttools.func.Tilings;
 import uk.ac.starlink.ttools.func.Times;
 import uk.ac.starlink.ttools.func.TrigDegrees;
+import uk.ac.starlink.ttools.func.URLs;
 import uk.ac.starlink.util.Loader;
 
 /**
@@ -142,6 +143,7 @@ public class JELUtils {
                 Tilings.class,
                 Times.class,
                 TrigDegrees.class,
+                URLs.class,
             } ) );
 
             /* Add classes specified by a system property. */
