@@ -104,11 +104,7 @@ public class MatchMapper implements TableMapper {
             "See <ref id='matchGroup'/> for more discussion.",
             "</li>",
             "</ul>",
-            "In the case of well-separated objects these modes will give",
-            "the same results.  For crowded fields however it will make",
-            "a difference which is chosen.",
-            "</p>",
-            "<p>Note that which rows actually appear in the output",
+            "Note that which rows actually appear in the output",
             "is also influenced by the", 
             "<code>" + new MultiJoinTypeParameter( "N" ).getName() + "</code>",
             "parameter.",
@@ -120,8 +116,8 @@ public class MatchMapper implements TableMapper {
         progressParam_ = new ProgressIndicatorParameter( "progress" );
     }
 
-    public Parameter[] getParameters() {
-        return new Parameter[] {
+    public Parameter<?>[] getParameters() {
+        return new Parameter<?>[] {
             mmodeParam_,
             irefParam_,
             matcherParam_,
