@@ -19,6 +19,7 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.Supplier;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import javax.swing.Icon;
@@ -28,9 +29,12 @@ import uk.ac.starlink.ttools.plot.Picture;
 import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
+import uk.ac.starlink.ttools.plot2.data.DataSpec;
 import uk.ac.starlink.ttools.plot2.data.DataStore;
+import uk.ac.starlink.ttools.plot2.data.TupleRunner;
 import uk.ac.starlink.ttools.plot2.data.TupleSequence;
 import uk.ac.starlink.ttools.plot2.paper.PaperType;
+import uk.ac.starlink.util.SplitCollector;
 
 /**
  * Miscellaneous utilities for use with the plotting classes.
@@ -52,6 +56,12 @@ public class PlotUtil {
         public boolean next() {
             return false;
         }
+        public TupleSequence split() {
+            return null;
+        }
+        public long splittableSize() {
+            return 0L;
+        }
         public long getRowIndex() {
             return -1L;
         }
@@ -105,6 +115,10 @@ public class PlotUtil {
     /** Level at which plot reports are logged. */
     private static final Level REPORT_LEVEL = Level.INFO;
 
+     /** Default SplitRunner for CoordSequences. */
+     public static SplitRunner<CoordSequence> COORD_RUNNER =
+         SplitRunner.createDefaultRunner();
+
     /**
      * Private constructor prevents instantiation.
      */
@@ -454,6 +468,24 @@ public class PlotUtil {
         };
     }
 
+    /**
+     * Convenience TupleRunner collection method using a DataStore and DataSpec.
+     * Default accumulator pooling policy is used (no pool).
+     *
+     * @param   collector  collector
+     * @param   dataSpec   data spec
+     * @param   dataStore   data store, supplying both the data and the runner
+     * @return   collected result
+     */
+    @Slow
+    public static <A> A tupleCollect( SplitCollector<TupleSequence,A> collector,
+                                      DataSpec dataSpec, DataStore dataStore ) {
+        return dataStore.getTupleRunner()
+              .collect( collector,
+                        () -> dataStore.getTupleSequence( dataSpec ) );
+    }
+
+
     /**
      * Extends existing range objects using range information
      * for a set of layers which have Cartesian (or similar) coordinates.
@@ -488,13 +520,14 @@ public class PlotUtil {
                          SubCloud.createPartialSubClouds( layers, true ) );
         PointCloud cloud = new PointCloud( subClouds );
 
-        /* Iterate over the represented points to mark out the basic
+        /* Collect values for the represented points to mark out the basic
          * range of data positions covered by the layers. */
-        for ( double[] dpos : cloud.createDataPosIterable( dataStore ) ) {
-            for ( int idim = 0; idim < nDataDim; idim++ ) {
-                ranges[ idim ].submit( dpos[ idim ] );
-            }
-        }
+        RangeCollector rangeCollector = new RangeCollector( nDataDim );
+        Range[] cloudRanges =
+            dataStore.getTupleRunner().coordRunner()
+                     .collect( rangeCollector, 
+                               cloud.createDataPosSupplier( dataStore ) );
+        rangeCollector.mergeRanges( ranges, cloudRanges );
 
         /* If any of the layers wants to supply non-data-position points
          * to mark out additional space, take account of those too. */
@@ -608,32 +641,26 @@ public class PlotUtil {
      * @param  surface  plot surface
      * @param  geom     maps data positions to graphics positions
      * @param  iPosCoord   coordinate index of positional coords in tseq
-     * @param  tseq     tuple sequence positioned at start
+     * @param  tupleSupplier    iterable over tuples
+     * @param  runner    manages tuple iteration
      * @param  point    reference graphics position
      * @return   object giving row index and distance;
      *           null is returned if no points are present
      */
     @Slow
-    public static IndicatedRow getClosestRow( Surface surface, DataGeom geom,
-                                              int iPosCoord, TupleSequence tseq,
-                                              Point2D point ) {
-        double[] dpos = new double[ surface.getDataDimCount() ];
-        Point2D.Double gp = new Point2D.Double();
-        long bestIndex = -1;
-        double bestDist2 = Double.POSITIVE_INFINITY;
-        while ( tseq.next() ) {
-            if ( geom.readDataPos( tseq, iPosCoord, dpos ) &&
-                 surface.dataToGraphics( dpos, true, gp ) ) {
-                double dist2 = gp.distanceSq( point );
-                if ( dist2 < bestDist2 ) {
-                    bestDist2 = dist2;
-                    bestIndex = tseq.getRowIndex();
-                }
-            }
-        }
-        return Thread.currentThread().isInterrupted() || bestIndex < 0
+    public static IndicatedRow
+            getClosestRow( Surface surface, DataGeom geom, int iPosCoord,
+                           Supplier<TupleSequence> tupleSupplier,
+                           TupleRunner runner, Point2D point ) {
+        IndexDist ixdist = 
+            runner
+           .collect( new ClosestCollector( surface, geom, iPosCoord, point ),
+                     tupleSupplier );
+        return ixdist.bestIndex_ < 0 ||
+               Thread.currentThread().isInterrupted()
              ? null
-             : new IndicatedRow( bestIndex, Math.sqrt( bestDist2 ) );
+             : new IndicatedRow( ixdist.bestIndex_,
+                                 Math.sqrt( ixdist.bestDist2_ ) );
     }
 
     /**     
@@ -1083,4 +1110,119 @@ public class PlotUtil {
                               base.width - insets.left - insets.right,
                               base.height - insets.top - insets.bottom );
     }
+
+    /**
+     * SplitCollector implementation that can accumulate range information
+     * from a CoordSequence.
+     */
+    private static class RangeCollector
+            implements SplitCollector<CoordSequence,Range[]> {
+
+        private final int ndim_;
+
+        /**
+         * Constructor.
+         *
+         * @param  ndim  number of range objects (data dimensions)
+         */
+        public RangeCollector( int ndim ) {
+            ndim_ = ndim;
+        }
+
+        public Range[] createAccumulator() {
+            Range[] ranges = new Range[ ndim_ ];
+            for ( int i = 0; i < ndim_; i++ ) {
+                ranges[ i ] = new Range();
+            }
+            return ranges;
+        }
+
+        public void accumulate( CoordSequence cseq, Range[] ranges ) {
+            double[] dpos = cseq.getCoords();
+            while ( cseq.next() ) {
+                for ( int idim = 0; idim < ndim_; idim++ ) {
+                    ranges[ idim ].submit( dpos[ idim ] );
+                }
+            }
+        }
+
+        public Range[] combine( Range[] ranges1, Range[] ranges2 ) {
+            mergeRanges( ranges1, ranges2 );
+            return ranges1;
+        }
+
+        /**
+         * Merges the content of the second range into the first one.
+         *
+         * @param  ranges0  first input range, modified on exit
+         * @param  ranges1  second input range, unmodified on exit
+         */
+        public void mergeRanges( Range[] ranges0, Range[] ranges1 ) {
+            for ( int i = 0; i < ndim_; i++ ) {
+                ranges0[ i ].extend( ranges1[ i ] );
+            }
+        }
+    }
+
+    /**
+     * Accumulator class for use with ClosestCollector.
+     * Aggregates the index of the current best row with its distance
+     * from the target position.
+     */
+    private static class IndexDist {
+        long bestIndex_ = -1;
+        double bestDist2_ = Double.POSITIVE_INFINITY;
+    }
+
+    /**
+     * SplitCollector implementation for determining the closest row
+     * to a given graphics position on a plotting surface.
+     */
+    private static class ClosestCollector
+            implements SplitCollector<TupleSequence,IndexDist> {
+
+        private final Surface surface_;
+        private final DataGeom geom_;
+        private final int iPosCoord_;
+        private final Point2D point_;
+
+        /**
+         * Constructor.
+         *
+         * @param  surface  plot surface
+         * @param  geom     maps data positions to graphics positions
+         * @param  iPosCoord   coordinate index of positional coords in tseq
+         * @param  point    reference graphics position
+         */
+        public ClosestCollector( Surface surface, DataGeom geom, int iPosCoord,
+                                 Point2D point ) {
+            surface_ = surface;
+            geom_ = geom;
+            iPosCoord_ = iPosCoord;
+            point_ = point;
+        }
+
+        public IndexDist createAccumulator() {
+            return new IndexDist();
+        }
+
+        public void accumulate( TupleSequence tseq, IndexDist acc ) {
+            double[] dpos = new double[ surface_.getDataDimCount() ];
+            Point2D.Double gp = new Point2D.Double();
+            while ( tseq.next() ) {
+                if ( geom_.readDataPos( tseq, iPosCoord_, dpos ) &&
+                     surface_.dataToGraphics( dpos, true, gp ) ) {
+                    double dist2 = gp.distanceSq( point_ );
+                    if ( dist2 < acc.bestDist2_ ) {
+                        acc.bestDist2_ = dist2;
+                        acc.bestIndex_ = tseq.getRowIndex();
+                    }
+                }
+            }
+        }
+
+        public IndexDist combine( IndexDist acc1, IndexDist acc2 ) {
+            return acc1.bestDist2_ <= acc2.bestDist2_ ? acc1 : acc2;
+        }
+    }
 }
