@@ -116,6 +116,7 @@ public class ColorConfigKey extends ChoiceConfigKey<Color> {
         Color[] colors = getOptionMap().values().toArray( new Color[ 0 ] );
         List<Specifier<Color>> specifiers = new ArrayList<Specifier<Color>>();
         specifiers.add( new ComboBoxSpecifier<Color>(
+                            Color.class,
                             new ColorComboBox( colors ) ) );
         specifiers.add( new ChooserColorSpecifier( colors[ 0 ] ) );
         Specifier<Color> basic =
@@ -207,10 +208,8 @@ public class ColorConfigKey extends ChoiceConfigKey<Color> {
             new LinkedHashMap<String,Color>( STANDARD_COLORS );
         map.remove( COLORNAME_LIGHTGREY );
         map.remove( COLORNAME_BLACK );
-        for ( Iterator<Map.Entry<String,Color>> it = map.entrySet().iterator();
-              it.hasNext(); ) {
-            Map.Entry entry = it.next();
-            if ( Color.WHITE.equals( entry.getValue() ) ) {
+        for ( Iterator<Color> it = map.values().iterator(); it.hasNext(); ) {
+            if ( Color.WHITE.equals( it.next() ) ) {
                 it.remove();
             }
         }
