@@ -5,6 +5,7 @@ import java.util.logging.Level;
 import java.util.logging.Logger;
 import uk.ac.starlink.table.DomainMapper;
 import uk.ac.starlink.table.RowSequence;
+import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.ValueInfo;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
 
@@ -20,9 +21,20 @@ import uk.ac.starlink.ttools.plot2.PlotUtil;
  */
 public class SimpleDataStoreFactory implements DataStoreFactory, DataStore {
 
+    private final TupleRunner runner_;
+
     private static Logger logger_ =
         Logger.getLogger( "uk.ac.starlink.ttools.plot2" );
 
+    /**
+     * Constructor.
+     *
+     * @param  runner  tuple runner dispensed with DataStores
+     */
+    public SimpleDataStoreFactory( TupleRunner runner ) {
+        runner_ = runner;
+    }
+
     public boolean hasData( DataSpec spec ) {
         return true;
     }
@@ -35,16 +47,26 @@ public class SimpleDataStoreFactory implements DataStoreFactory, DataStore {
     }
 
     public TupleSequence getTupleSequence( DataSpec spec ) {
-        try {
-            RowSequence rseq = spec.getSourceTable().getRowSequence();
-            return new SimpleTupleSequence( spec, rseq );
+        StarTable table = spec.getSourceTable();
+        if ( table.isRandom() && table.getRowCount() > 2 ) {
+            return new RandomSimpleTupleSequence( spec, table );
         }
-        catch ( IOException e ) {
-            logger_.log( Level.WARNING, "Error reading plot data", e );
-            return PlotUtil.EMPTY_TUPLE_SEQUENCE;
+        else {
+            try {
+                RowSequence rseq = spec.getSourceTable().getRowSequence();
+                return new SequentialSimpleTupleSequence( spec, rseq );
+            }
+            catch ( IOException e ) {
+                logger_.log( Level.WARNING, "Error reading plot data", e );
+                return PlotUtil.EMPTY_TUPLE_SEQUENCE;
+            }
         }
     }
 
+    public TupleRunner getTupleRunner() {
+        return runner_;
+    }
+
     /**
      * Utility method to work out the domain mappers for a given
      * coordinate of a DataSpec.
@@ -80,53 +102,33 @@ public class SimpleDataStoreFactory implements DataStoreFactory, DataStore {
     }
 
     /**
-     * TupleSequence implementation for use with SimpleDataStoreFactory.
+     * Abstract superclass for tuple sequences dispatched by this factory.
      * It simply reads the user data every time and converts it to
      * storage format as the sequence iterates.
      */
-    private static class SimpleTupleSequence implements TupleSequence {
-        private final DataSpec spec_;
-        private final UserDataReader reader_;
-        private final RowSequence baseSeq_;
-        private final DomainMapper[][] mappers_;
-        private long irow_ = -1;
-        private boolean failed_;
+    private static abstract class SimpleTupleSequence implements TupleSequence {
+
+        final DataSpec spec_;
+        final UserDataReader reader_;
+        final DomainMapper[][] mappers_;
+        RowSequence rowseq_;
+        long irow_;
+        boolean failed_;
 
         /**
-         * Constructor.
+         * Constructor.  Subclasses must initialise the rowseq_ member.
          *
          * @param   spec  data specification
-         * @param   rseq  row sequence from spec's source table
          */
-        SimpleTupleSequence( DataSpec spec, RowSequence rseq ) {
+        SimpleTupleSequence( DataSpec spec ) {
             spec_ = spec;
             reader_ = spec.createUserDataReader();
-            baseSeq_ = rseq;
             int nc = spec.getCoordCount();
             mappers_ = new DomainMapper[ nc ][];
             for ( int ic = 0; ic < nc; ic++ ) {
                 mappers_[ ic ] = getUserCoordMappers( spec, ic );
             }
-        }
-
-        public boolean next() {
-            try {
-                while ( ! failed_ && baseSeq_.next() ) {
-                    if ( reader_.getMaskFlag( baseSeq_, ++irow_ ) ) {
-                        return true;
-                    }
-                }
-            }
-            catch ( IOException e ) {
-                logError( e );
-            }
-            try {
-                baseSeq_.close();
-            }
-            catch ( IOException e ) {
-                // ignore
-            }
-            return false;
+            irow_ = -1;
         }
 
         public long getRowIndex() {
@@ -136,7 +138,7 @@ public class SimpleDataStoreFactory implements DataStoreFactory, DataStore {
         public Object getObjectValue( int icol ) {
             try {
                 Object[] userCoords =
-                    reader_.getUserCoordValues( baseSeq_, irow_, icol );
+                    reader_.getUserCoordValues( rowseq_, irow_, icol );
                 Object value =
                     spec_.getCoord( icol )
                          .inputToStorage( userCoords, mappers_[ icol ] );
@@ -171,10 +173,137 @@ public class SimpleDataStoreFactory implements DataStoreFactory, DataStore {
             return Boolean.TRUE.equals( getObjectValue( icol ) );
         }
 
-        private void logError( IOException err ) {
+        void logError( IOException err ) {
             failed_ = true;
             logger_.log( Level.WARNING,
                          "Error reading plot data - truncating sequence", err );
         }
     }
+
+    /**
+     * SimpleTupleSequence concrete implementation for sequential data;
+     * there is no attempt to split.
+     */
+    private static class SequentialSimpleTupleSequence
+            extends SimpleTupleSequence {
+
+        /**
+         * Constructor.
+         *
+         * @param   spec  data specification
+         * @param   rseq  row sequence from spec's source table
+         */
+        SequentialSimpleTupleSequence( DataSpec spec, RowSequence rseq ) {
+            super( spec );
+            rowseq_ = rseq;
+        }
+
+        public boolean next() {
+            try {
+                while ( ! failed_ && rowseq_.next() ) {
+                    if ( reader_.getMaskFlag( rowseq_, ++irow_ ) ) {
+                        return true;
+                    }
+                }
+            }
+            catch ( IOException e ) {
+                logError( e );
+            }
+            try {
+                rowseq_.close();
+            }
+            catch ( IOException e ) {
+                // ignore
+            }
+            return false;
+        }
+
+        public TupleSequence split() {
+            return null;
+        }
+
+        public long splittableSize() {
+            return spec_.getSourceTable().getRowCount();
+        }
+    }
+
+    /**
+     * SimpleTupleSequence concrete implementation for random access data;
+     * it can be split.
+     */
+    private static class RandomSimpleTupleSequence extends SimpleTupleSequence {
+
+        private final StarTable table_;
+        private long nrow_;
+
+        /**
+         * Constructor for external use.
+         *
+         * @param   spec  data specification
+         * @param   table  source table
+         */
+        RandomSimpleTupleSequence( DataSpec spec, StarTable table ) {
+            this( spec, table, -1, table.getRowCount() );
+        }
+
+        /**
+         * Constructor for internal use (recursion).
+         *
+         * @param   spec  data specification
+         * @param   table  source table
+         * @param   irow   initial row index
+         * @param   nrow   first index after iteration run
+         */
+        RandomSimpleTupleSequence( DataSpec spec, StarTable table,
+                                   long irow, long nrow ) {
+            super( spec );
+            rowseq_ = new RowSequence() {
+                public Object getCell( int icol ) throws IOException {
+                    return table_.getCell( irow_, icol );
+                }
+                public Object[] getRow() throws IOException {
+                    return table_.getRow( irow_ );
+                }
+                public boolean next() {
+                    return ++irow_ < nrow_;
+                }
+                public void close() {
+                }
+            };
+            table_ = table;
+            irow_ = irow;
+            nrow_ = nrow;
+        }
+
+        public boolean next() {
+            try {
+                while ( ! failed_ && irow_ < nrow_ - 1 ) {
+                    if ( reader_.getMaskFlag( rowseq_, ++irow_ ) ) {
+                        return true;
+                    }
+                }
+            }
+            catch ( IOException e ) {
+                logError( e );
+            }
+            return false;
+        }
+
+        public TupleSequence split() {
+            if ( nrow_ - irow_ > 2 ) {
+                long mid = ( irow_ + nrow_ ) / 2;
+                TupleSequence split =
+                    new RandomSimpleTupleSequence( spec_, table_, irow_, mid );
+                irow_ = mid - 1;
+                return split;
+            }
+            else {
+                return null;
+            }
+        }
+
+        public long splittableSize() {
+            return nrow_ - irow_;
+        }
+    }
 }
