@@ -8,6 +8,7 @@ import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.Captioner;
 import uk.ac.starlink.ttools.plot2.Navigator;
 import uk.ac.starlink.ttools.plot2.PlotLayer;
+import uk.ac.starlink.ttools.plot2.PlotMetric;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.Surface;
@@ -291,10 +292,10 @@ public class CubeSurfaceFactory
                               p.captioner_, p.frame_, p.minor_, p.antialias_ );
     }
 
-    public ConfigKey[] getProfileKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getProfileKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
         if ( ! isIso_ ) {
-            list.addAll( Arrays.asList( new ConfigKey[] {
+            list.addAll( Arrays.asList( new ConfigKey<?>[] {
                 XLOG_KEY,
                 YLOG_KEY,
                 ZLOG_KEY,
@@ -310,17 +311,17 @@ public class CubeSurfaceFactory
             } ) );
         }
         else {
-            list.addAll( Arrays.asList( new ConfigKey[] {
+            list.addAll( Arrays.asList( new ConfigKey<?>[] {
                 ISOCROWD_KEY,
             } ) );
         }
-        list.addAll( Arrays.asList( new ConfigKey[] {
+        list.addAll( Arrays.asList( new ConfigKey<?>[] {
             FRAME_KEY, 
             StyleKeys.MINOR_TICKS,
             StyleKeys.GRID_ANTIALIAS,
         } ) );
         list.addAll( Arrays.asList( StyleKeys.CAPTIONER.getKeys() ) );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public Profile createProfile( ConfigMap config ) {
@@ -347,29 +348,29 @@ public class CubeSurfaceFactory
                             minor, antialias );
     }
 
-    public ConfigKey[] getAspectKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getAspectKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
         if ( isIso_ ) {
-            list.addAll( Arrays.asList( new ConfigKey[] {
+            list.addAll( Arrays.asList( new ConfigKey<?>[] {
                 XC_KEY, YC_KEY, ZC_KEY,
                 SCALE_KEY,
             } ) );
         }
         else {
-            list.addAll( Arrays.asList( new ConfigKey[] {
+            list.addAll( Arrays.asList( new ConfigKey<?>[] {
                 XMIN_KEY, XMAX_KEY, XSUBRANGE_KEY,
                 YMIN_KEY, YMAX_KEY, YSUBRANGE_KEY,
                 ZMIN_KEY, ZMAX_KEY, ZSUBRANGE_KEY,
             } ) );
         }
-        list.addAll( Arrays.asList( new ConfigKey[] {
+        list.addAll( Arrays.asList( new ConfigKey<?>[] {
             PHI_KEY,
             THETA_KEY,
             PSI_KEY,
             ZOOM_KEY,
             XOFF_KEY, YOFF_KEY,
         } ) );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public boolean useRanges( Profile profile, ConfigMap config ) {
@@ -403,10 +404,13 @@ public class CubeSurfaceFactory
                                DataStore dataStore ) {
         boolean[] logFlags = profile.getLogFlags();
         assert logFlags.length == 3;
-        return PlotUtil.readCoordinateRanges( layers, 3, logFlags, dataStore );
+        Range[] ranges = new Range[] { new Range(), new Range(), new Range() };
+        PlotUtil.extendCoordinateRanges( layers, ranges, logFlags, true,
+                                         dataStore );
+        return ranges;
     }
 
-    public ConfigKey[] getNavigatorKeys() {
+    public ConfigKey<?>[] getNavigatorKeys() {
         return CubeNavigator.getConfigKeys( isIso_ );
     }
 
@@ -414,6 +418,13 @@ public class CubeSurfaceFactory
         return CubeNavigator.createNavigator( isIso_, navConfig );
     }
 
+    /**
+     * Returns null.
+     */
+    public PlotMetric getPlotMetric() {
+        return null;
+    }
+
     /**
      * Attempts to determine axis data limits from profile and configuration,
      * but not ranging, information.  If not enough information is supplied,
