@@ -29,6 +29,7 @@ import uk.ac.starlink.ttools.plot2.layer.MultiPointForm;
 import uk.ac.starlink.ttools.plot2.layer.PairLinkForm;
 import uk.ac.starlink.ttools.plot2.layer.PlaneCorrelationCoordSet;
 import uk.ac.starlink.ttools.plot2.layer.PlaneEllipseCoordSet;
+import uk.ac.starlink.ttools.plot2.layer.PolygonForms;
 import uk.ac.starlink.ttools.plot2.layer.SizeForm;
 import uk.ac.starlink.ttools.plot2.layer.ShapeForm;
 import uk.ac.starlink.ttools.plot2.layer.ShapeMode;
@@ -47,34 +48,38 @@ import uk.ac.starlink.ttools.plot2.paper.PaperTypeSelector;
  * @author   Mark Taylor
  * @since    19 Feb 2013
  */
-public class PlanePlotType implements PlotType {
+public class PlanePlotType
+        implements PlotType<PlaneSurfaceFactory.Profile,PlaneAspect> {
 
-    private static final SurfaceFactory SURFACE_FACTORY =
-        new PlaneSurfaceFactory();
     private static PlaneDataGeom DATAGEOM = PlaneDataGeom.INSTANCE;
     private static final PlanePlotType INSTANCE =
-        new PlanePlotType( createDefaultPlotters() );
-    private final Plotter[] plotters_;
+        new PlanePlotType( createDefaultPlotters(), true );
+    private final PlaneSurfaceFactory surfFact_;
+    private final Plotter<?>[] plotters_;
 
     /**
      * Constructor.
      *
      * @param  plotters  available plotters for use with this plot type
+     * @param  has2dMetric  true if it may make sense to measure distances
+     *                      that are not parallel to either axis
      */
-    public PlanePlotType( Plotter[] plotters ) {
+    public PlanePlotType( Plotter<?>[] plotters, boolean has2dMetric ) {
         plotters_ = plotters;
+        surfFact_ = new PlaneSurfaceFactory( has2dMetric );
     }
 
     public DataGeom[] getPointDataGeoms() {
         return new DataGeom[] { DATAGEOM };
     }
 
-    public Plotter[] getPlotters() {
+    public Plotter<?>[] getPlotters() {
         return plotters_.clone();
     }
 
-    public SurfaceFactory getSurfaceFactory() {
-        return SURFACE_FACTORY;
+    public SurfaceFactory<PlaneSurfaceFactory.Profile,PlaneAspect>
+                          getSurfaceFactory() {
+        return surfFact_;
     }
 
     public PaperTypeSelector getPaperTypeSelector() {
@@ -99,14 +104,14 @@ public class PlanePlotType implements PlotType {
      *
      * @return  plane plotter list
      */
-    private static Plotter[] createDefaultPlotters() {
+    private static Plotter<?>[] createDefaultPlotters() {
         Coord[] coords = DATAGEOM.getPosCoords();
         String[] axisNames = new String[ coords.length ];
         for ( int i = 0; i < coords.length; i++ ) {
             axisNames[ i ] = ((FloatingCoord) coords[ i ])
                             .getInput().getMeta().getLongName();
         };
-        List<Plotter> list = new ArrayList<Plotter>();
+        List<Plotter<?>> list = new ArrayList<Plotter<?>>();
         ShapeForm[] forms = new ShapeForm[] {
             MarkForm.SINGLE,
             SizeForm.getInstance(),
@@ -123,12 +128,15 @@ public class PlanePlotType implements PlotType {
             PlaneCorrelationCoordSet.createForm(),
             PairLinkForm.getInstance(),
             MarkForm.PAIR,
+            PolygonForms.QUAD,
+            MarkForm.QUAD,
+            PolygonForms.ARRAY,
         };
-        Plotter[] shapePlotters =
+        Plotter<?>[] shapePlotters =
             ShapePlotter.createShapePlotters( forms, ShapeMode.MODES_2D );
         list.addAll( Arrays.asList( shapePlotters ) );
         PerUnitConfigKey<Unit> unitKey = null;
-        list.addAll( Arrays.asList( new Plotter[] {
+        list.addAll( Arrays.asList( new Plotter<?>[] {
             new LinePlotter(),
             new LinearFitPlotter( true ),
             new LabelPlotter(),
@@ -144,6 +152,6 @@ public class PlanePlotType implements PlotType {
             new Stats1Plotter( PlaneDataGeom.X_COORD, true, unitKey ),
             FunctionPlotter.PLANE,
         } ) );
-        return list.toArray( new Plotter[ 0 ] );
+        return list.toArray( new Plotter<?>[ 0 ] );
     }
 }
