@@ -15,9 +15,13 @@ import uk.ac.starlink.ttools.plot.Style;
 import uk.ac.starlink.ttools.plot2.Axis;
 import uk.ac.starlink.ttools.plot2.LayerOpt;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
+import uk.ac.starlink.ttools.plot2.Ranger;
 import uk.ac.starlink.ttools.plot2.ReportMap;
 import uk.ac.starlink.ttools.plot2.Scaler;
 import uk.ac.starlink.ttools.plot2.Scaling;
+import uk.ac.starlink.ttools.plot2.Scalings;
+import uk.ac.starlink.ttools.plot2.Span;
+import uk.ac.starlink.ttools.plot2.Subrange;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
 import uk.ac.starlink.ttools.plot2.config.ConfigMeta;
@@ -107,8 +111,8 @@ public class DensogramPlotter
         } );
     }
 
-    public ConfigKey[] getStyleKeys() {
-        List<ConfigKey> list = new ArrayList<ConfigKey>();
+    public ConfigKey<?>[] getStyleKeys() {
+        List<ConfigKey<?>> list = new ArrayList<ConfigKey<?>>();
         list.add( StyleKeys.COLOR );
         list.add( SMOOTHSIZER_KEY );
         list.add( KERNEL_KEY );
@@ -116,7 +120,7 @@ public class DensogramPlotter
         list.add( StyleKeys.CUMULATIVE );
         list.add( EXTENT_KEY );
         list.add( POSITION_KEY );
-        return list.toArray( new ConfigKey[ 0 ] );
+        return list.toArray( new ConfigKey<?>[ 0 ] );
     }
 
     public DensoStyle createStyle( ConfigMap config ) {
@@ -129,8 +133,8 @@ public class DensogramPlotter
         int extent = config.get( EXTENT_KEY );
         double position = config.get( POSITION_KEY );
         return new DensoStyle( baseColor, ramp.getShader(), ramp.getScaling(),
-                               kernelShape, combiner, sizer, cumul, extent,
-                               position );
+                               ramp.getDataClip(), kernelShape, combiner,
+                               sizer, cumul, extent, position );
     }
 
     protected void paintBins( PlanarSurface surface, BinArray binArray,
@@ -160,24 +164,20 @@ public class DensogramPlotter
         int np = ixhi - ixlo;
 
         /* Get range. */
-        double ymin = Double.POSITIVE_INFINITY;
-        double ymax = Double.NEGATIVE_INFINITY;
+        Scaling scaling = style.scaling_;
+        Ranger ranger = Scalings.createRanger( new Scaling[] { scaling } );
         for ( int ip = 0; ip < np; ip++ ) {
-            int ix = ixlo + ip;
-            double dy = bins[ ix ];
-            if ( ! Double.isNaN( dy ) ) {
-                ymin = Math.min( ymin, dy );
-                ymax = Math.max( ymax, dy );
-            }
+            ranger.submitDatum( bins[ ixlo + ip ] );
         }
+        Span span = ranger.createSpan();
 
         /* Do the painting. */
-        if ( ymin < ymax ) {
-            Scaler scaler = style.scaling_.createScaler( ymin, ymax );
+        if ( span.getHigh() > span.getLow() ) {
+            Scaler scaler = span.createScaler( scaling, style.dataclip_ );
             float[] baseRgba = style.baseColor_.getRGBComponents( null );
             float[] rgba = new float[ 4 ];
             Color color0 = g.getColor();
-            boolean isLog = style.scaling_.isLogLike();
+            boolean isLog = scaler.isLogLike();
             for ( int ip = 0; ip < np; ip++ ) {
                 int ix = ixlo + ip;
                 int gx = binArray.getGraphicsCoord( ix );
@@ -243,6 +243,7 @@ public class DensogramPlotter
         final Color baseColor_;
         final Shader shader_;
         final Scaling scaling_;
+        final Subrange dataclip_;
         final Kernel1dShape kernelShape_;
         final Combiner combiner_;
         final BinSizer sizer_;
@@ -256,6 +257,7 @@ public class DensogramPlotter
          * @param  baseColor   base colour
          * @param  shader    colour ramp shader
          * @param  scaling   colour ramp scaling function
+         * @param  dataclip  input value subrange
          * @param  kernelShape   smoothing kernel shape
          * @param  combiner   pixel bin aggregation mode
          * @param  sizer    smoothing width specification
@@ -264,12 +266,13 @@ public class DensogramPlotter
          * @param  position   fractional location of density bar (0..1)
          */
         public DensoStyle( Color baseColor, Shader shader, Scaling scaling,
-                           Kernel1dShape kernelShape, Combiner combiner,
-                           BinSizer sizer, boolean cumul, int extent,
-                           double position ) {
+                           Subrange dataclip, Kernel1dShape kernelShape,
+                           Combiner combiner, BinSizer sizer, boolean cumul,
+                           int extent, double position ) {
             baseColor_ = baseColor;
             shader_ = shader;
             scaling_ = scaling;
+            dataclip_ = dataclip;
             kernelShape_ = kernelShape;
             combiner_ = combiner;
             sizer_ = sizer;
@@ -288,6 +291,7 @@ public class DensogramPlotter
             code = 23 * code + baseColor_.hashCode();
             code = 23 * code + shader_.hashCode();
             code = 23 * code + scaling_.hashCode();
+            code = 23 * code + dataclip_.hashCode();
             code = 23 * code + kernelShape_.hashCode();
             code = 23 * code + combiner_.hashCode();
             code = 23 * code + sizer_.hashCode();
@@ -304,6 +308,7 @@ public class DensogramPlotter
                 return this.baseColor_.equals( other.baseColor_ )
                     && this.shader_.equals( other.shader_ )
                     && this.scaling_.equals( other.scaling_ )
+                    && this.dataclip_.equals( other.dataclip_ )
                     && this.kernelShape_.equals( other.kernelShape_ )
                     && this.combiner_.equals( other.combiner_ )
                     && this.sizer_.equals( other.sizer_ )
