@@ -12,7 +12,6 @@ import javax.swing.Icon;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
 import uk.ac.starlink.ttools.plot.MarkShape;
 import uk.ac.starlink.ttools.plot.MarkStyle;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.AuxReader;
 import uk.ac.starlink.ttools.plot2.AuxScale;
 import uk.ac.starlink.ttools.plot2.DataGeom;
@@ -20,6 +19,7 @@ import uk.ac.starlink.ttools.plot2.Glyph;
 import uk.ac.starlink.ttools.plot2.Pixer;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
 import uk.ac.starlink.ttools.plot2.PointCloud;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.SubCloud;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
@@ -48,12 +48,18 @@ public abstract class MarkForm implements ShapeForm {
     private final String name_;
     private final Icon icon_;
 
+    /** Minimum size of marker in legend, even if plot size is smaller. */
+    public static final int MIN_LEGEND_SIZE = 2;
+
     /** MarkForm instance for a single point per tuple. */
     public static final MarkForm SINGLE = createMarkForm( 1 );
 
     /** MarkForm instance for two points per tuple. */
     public static final MarkForm PAIR = createMarkForm( 2 );
 
+    /** MarkForm instance for four points per tuple. */
+    public static final MarkForm QUAD = createMarkForm( 4 );
+
     private static final Color DUMMY_COLOR = Color.GRAY;
 
     /**
@@ -118,8 +124,8 @@ public abstract class MarkForm implements ShapeForm {
     public static MarkForm createMarkForm( final int npos ) {
         if ( npos == 1 ) {
             return new MarkForm( 1, "Mark", ResourceIcon.FORM_MARK ) {
-                public ConfigKey[] getConfigKeys() {
-                    return new ConfigKey[] {
+                public ConfigKey<?>[] getConfigKeys() {
+                    return new ConfigKey<?>[] {
                         StyleKeys.MARK_SHAPE,
                         StyleKeys.SIZE,
                     };
@@ -132,20 +138,30 @@ public abstract class MarkForm implements ShapeForm {
             };
         }
         else {
-            return new MarkForm( npos, "Mark" + npos,
-                                 npos == 2 ? ResourceIcon.FORM_MARKS2
-                                           : ResourceIcon.FORM_MARKS3 ) {
+            final Icon icon;
+            if ( npos == 2 ) {
+                icon = ResourceIcon.FORM_MARKS2;
+            }
+            else if ( npos == 3 ) {
+                icon = ResourceIcon.FORM_MARKS3;
+            }
+            else {
+                icon = ResourceIcon.FORM_MARKS4;
+            }
+            return new MarkForm( npos, "Mark" + npos, icon ) {
                 public String getFormDescription() {
                     return PlotUtil.concatLines( new String[] {
-                        "<p>Plots " + npos + "similar markers",
+                        "<p>Plots " + npos + " similar markers",
                         "of fixed size and shape",
                         "representing " + npos + " separate positions",
                         "from the same input table row.",
+                        "This is a convenience option that can be used with",
+                        "other plot layers based on " + npos + " positions.",
                         "</p>",
                     } );
                 }
-                public ConfigKey[] getConfigKeys() {
-                    return new ConfigKey[] {
+                public ConfigKey<?>[] getConfigKeys() {
+                    return new ConfigKey<?>[] {
                         StyleKeys.MARK_SHAPE,
                         StyleKeys.SIZE,
                     };
@@ -195,6 +211,27 @@ public abstract class MarkForm implements ShapeForm {
                          : shape.getStyle( DUMMY_COLOR, size );
     }
 
+    /**
+     * Creates a MarkStyle for use in generating legend images.
+     * This may be limited to a more visible size than the style used
+     * for plotting the actual data.
+     *
+     * @param  shape  marker shape
+     * @param  size   marker size in pixels
+     * @return   marker style
+     */
+    private static MarkStyle createLegendMarkStyle( MarkShape shape,
+                                                    int size ) {
+        if ( size == 0 ) {
+            return MarkShape.FILLED_SQUARE
+                  .getStyle( DUMMY_COLOR, MIN_LEGEND_SIZE );
+        }
+        else {
+            return shape
+                  .getStyle( DUMMY_COLOR, Math.max( size, MIN_LEGEND_SIZE ) );
+        }
+    }
+
     /**
      * Creates a Glyph representing a marker.
      *
@@ -240,7 +277,7 @@ public abstract class MarkForm implements ShapeForm {
      * @return   legend icon
      */
     public static Icon createLegendIcon( MarkShape shape, int size ) {
-        final MarkStyle style = createMarkStyle( shape, size );
+        final MarkStyle style = createLegendMarkStyle( shape, size );
         final Icon baseIcon = style.getLegendIcon();
         final int width = baseIcon.getIconWidth();
         final int height = baseIcon.getIconHeight();
@@ -270,7 +307,7 @@ public abstract class MarkForm implements ShapeForm {
      */
     private static Icon createMultiLegendIcon( MarkShape shape, int size,
                                                final int npos ) {
-        final MarkStyle style = createMarkStyle( shape, size );
+        final MarkStyle style = createLegendMarkStyle( shape, size );
         return new MultiPosIcon( npos ) {
             protected void paintPositions( Graphics g, Point[] positions ) {
                 for ( int ip = 0; ip < npos; ip++ ) {
@@ -332,7 +369,7 @@ public abstract class MarkForm implements ShapeForm {
         }
 
         public Object calculateBinPlan( Surface surface, DataGeom geom,
-                                        Map<AuxScale,Range> auxRanges,
+                                        Map<AuxScale,Span> auxSpans,
                                         DataStore dataStore, DataSpec dataSpec,
                                         Object[] knownPlans ) {
             return BinPlan
@@ -407,7 +444,7 @@ public abstract class MarkForm implements ShapeForm {
 
         public ShapePainter create2DPainter( final Surface surface,
                                              final DataGeom geom,
-                                             Map<AuxScale,Range> auxRanges,
+                                             Map<AuxScale,Span> auxSpans,
                                              final PaperType2D paperType ) {
             final double[] dpos = new double[ surface.getDataDimCount() ];
             final Point2D.Double gp = new Point2D.Double();
@@ -425,7 +462,7 @@ public abstract class MarkForm implements ShapeForm {
 
         public ShapePainter create3DPainter( final CubeSurface surface,
                                              final DataGeom geom,
-                                             Map<AuxScale,Range> auxRanges,
+                                             Map<AuxScale,Span> auxSpans,
                                              final PaperType3D paperType ) {
             final double[] dpos = new double[ surface.getDataDimCount() ];
             final GPoint3D gp = new GPoint3D();
@@ -484,7 +521,7 @@ public abstract class MarkForm implements ShapeForm {
 
         public ShapePainter create2DPainter( final Surface surface,
                                              final DataGeom geom,
-                                             Map<AuxScale,Range> auxRanges,
+                                             Map<AuxScale,Span> auxSpans,
                                              final PaperType2D paperType ) {
             final double[] dpos = new double[ surface.getDataDimCount() ];
             final Point2D.Double gp = new Point2D.Double();
@@ -505,7 +542,7 @@ public abstract class MarkForm implements ShapeForm {
 
         public ShapePainter create3DPainter( final CubeSurface surface,
                                              final DataGeom geom,
-                                             Map<AuxScale,Range> auxRanges,
+                                             Map<AuxScale,Span> auxSpans,
                                              final PaperType3D paperType ) {
             final double[] dpos = new double[ surface.getDataDimCount() ];
             final GPoint3D gp = new GPoint3D();
