@@ -9,12 +9,12 @@ import java.util.HashMap;
 import java.util.Map;
 import javax.swing.Icon;
 import uk.ac.starlink.ttools.gui.ResourceIcon;
-import uk.ac.starlink.ttools.plot.Range;
 import uk.ac.starlink.ttools.plot2.AuxReader;
 import uk.ac.starlink.ttools.plot2.AuxScale;
 import uk.ac.starlink.ttools.plot2.DataGeom;
 import uk.ac.starlink.ttools.plot2.Glyph;
 import uk.ac.starlink.ttools.plot2.PlotUtil;
+import uk.ac.starlink.ttools.plot2.Span;
 import uk.ac.starlink.ttools.plot2.Surface;
 import uk.ac.starlink.ttools.plot2.config.ConfigKey;
 import uk.ac.starlink.ttools.plot2.config.ConfigMap;
@@ -36,7 +36,7 @@ import uk.ac.starlink.ttools.plot2.paper.PaperType3D;
 public class PairLinkForm implements ShapeForm {
 
     private static final PairLinkForm instance_ = new PairLinkForm();
-    private static final LineXYShape LINE_SHAPE = LineXYShape.getInstance();
+    private static final LineXYShape LINE_SHAPE = LineXYShape.INSTANCE;
 
     /**
      * Private constructor prevents instantiation.
@@ -68,8 +68,8 @@ public class PairLinkForm implements ShapeForm {
         return new Coord[ 0 ];
     }
 
-    public ConfigKey[] getConfigKeys() {
-        return new ConfigKey[] {
+    public ConfigKey<?>[] getConfigKeys() {
+        return new ConfigKey<?>[] {
         };
     }
 
@@ -182,7 +182,7 @@ public class PairLinkForm implements ShapeForm {
 
         public ShapePainter create2DPainter( final Surface surface,
                                              final DataGeom geom,
-                                             Map<AuxScale,Range> auxRanges,
+                                             Map<AuxScale,Span> auxSpans,
                                              final PaperType2D paperType ) {
             int ndim = surface.getDataDimCount();
             final double[] dpos1 = new double[ ndim ];
@@ -204,7 +204,8 @@ public class PairLinkForm implements ShapeForm {
                          PlotUtil.isPointFinite( gp1 ) &&
                          geom.readDataPos( tuple, npc, dpos2 ) &&
                          surface.dataToGraphics( dpos2, false, gp2 ) &&
-                         PlotUtil.isPointFinite( gp2 ) ) {
+                         PlotUtil.isPointFinite( gp2 ) &&
+                         surface.isContinuousLine( dpos1, dpos2 ) ) {
                         PlotUtil.quantisePoint( gp1, gp1i );
                         PlotUtil.quantisePoint( gp2, gp2i );
                         if ( lineMightCross( bounds, gp1i, gp2i ) ) {
@@ -220,7 +221,7 @@ public class PairLinkForm implements ShapeForm {
 
         public ShapePainter create3DPainter( final CubeSurface surface,
                                              final DataGeom geom,
-                                             Map<AuxScale,Range> auxRanges,
+                                             Map<AuxScale,Span> auxSpans,
                                              final PaperType3D paperType ) {
             int ndim = surface.getDataDimCount();
             final double[] dpos1 = new double[ ndim ];
