@@ -8,10 +8,10 @@ import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import uk.ac.starlink.vo.ColumnMeta;
-import uk.ac.starlink.vo.EndpointSet;
 import uk.ac.starlink.vo.ForeignMeta;
 import uk.ac.starlink.vo.SchemaMeta;
 import uk.ac.starlink.vo.TableMeta;
+import uk.ac.starlink.vo.TapService;
 
 /**
  * Validation stage for checking that two sets of table metadata
@@ -50,7 +50,7 @@ public class CompareMetadataStage implements Stage {
         return "Compare table metadata from " + srcDesc1_ + " and " + srcDesc2_;
     }
 
-    public void run( Reporter reporter, EndpointSet endpointSet ) {
+    public void run( Reporter reporter, TapService tapService ) {
         SchemaMeta[] smetas1 = metaHolder1_.getTableMetadata();
         SchemaMeta[] smetas2 = metaHolder2_.getTableMetadata();
         if ( smetas1 == null || smetas2 == null ) {
@@ -196,8 +196,8 @@ public class CompareMetadataStage implements Stage {
         }
 
         /* Report on discrepancies. */
-        List<String> extras1 = new ArrayList( names1 );
-        List<String> extras2 = new ArrayList( names2 );
+        List<String> extras1 = new ArrayList<String>( names1 );
+        List<String> extras2 = new ArrayList<String>( names2 );
         extras1.removeAll( names2 );
         extras2.removeAll( names1 );
         String contextString = context == null ? "" : " from " + context;
@@ -231,7 +231,7 @@ public class CompareMetadataStage implements Stage {
         }
 
         /* Calculate and return intersection set. */
-        List<String> intersect = new ArrayList( names1 );
+        List<String> intersect = new ArrayList<String>( names1 );
         intersect.retainAll( names2 );
         return intersect;
     }
@@ -274,7 +274,7 @@ public class CompareMetadataStage implements Stage {
             return false;
         }
         dt1 = stripAdqlType( dt1 );
-        if ( dt1.equals( dt2 ) ) {
+        if ( dt1.equalsIgnoreCase( dt2 ) ) {
             return true;
         }
         if ( dt1.equals( "SMALLINT" ) && dt2.equals( "short" ) ||
