@@ -39,8 +39,11 @@ public enum FixedCode implements ReportCode {
     E_CLRD( "Unexpected columns in result" ),
     E_CNAM( "Query/result column name mismatch" ),
     E_CNID( "Column name not ADQL identifier" ),
+    E_CPIF( "No TAP capability std interface declared" ),
     E_CPIO( "Capabilities metadata read error" ),
     E_CPSX( "Capabilities metadata parse failure" ),
+    E_CPT1( "No unique TAP capability element" ),
+    E_CPTV( "TAP version declaration mismatch" ),
     E_CRSV( "Column name ADQL reserved word" ),
     E_CTNO( "Continuation document not found" ),
     E_CTYP( "Declared/result type mismatch" ),
@@ -48,6 +51,7 @@ public enum FixedCode implements ReportCode {
     E_CUNI( "Unit mismatch" ),
     E_CUTP( "Utype mismatch" ),
     E_DCER( "Document read error" ),
+    E_DECO( "Non-303 response to job deletion" ),
     E_DEHT( "Bad HTTP job connection" ),
     E_DEMO( "Job deletion failure" ),
     E_DENO( "Non-404 response for deleted job" ),
@@ -56,10 +60,12 @@ public enum FixedCode implements ReportCode {
     E_DFSF( "Duff query result parse failure" ),
     E_DNST( "Missing QUERY_STATUS from duff query" ),
     E_DQUS( "Unknown value for QUERY_STATUS" ),
+    E_DSUC( "Non-error return from duff query" ),
     E_ELDF( "Wrong top-level element" ),
     E_EST1( "Multiple QUERY_STATUS INFOs" ),
     E_EURL( "HTTP error" ),
     E_EXCH( "Bad continuation href attribute" ),
+    E_EXDH( "Examples declaration/presence mismatch" ),
     E_EXCR( "Bad continuation resource attribute" ),
     E_EXIO( "Error reading examples document" ),
     E_EXPA( "Badly formed XML examples document" ),
@@ -128,6 +134,8 @@ public enum FixedCode implements ReportCode {
     E_RRTO( "Row limit exceeded" ),
     E_RUPH( "Bad phase for started job" ),
     E_SCIO( "Error reading TAP_SCHEMA.schemas table" ),
+    E_SPPA( "Spurious query parameter breaks query" ),
+    E_TADH( "Tables endpoint declaration/presence mismatch" ),
     E_TBIO( "Error reading TAP_SCHEMA.tables table" ),
     E_TMCD( "Upload result column value mismatch" ),
     E_TMCN( "Upload result column name mismatch" ),
@@ -136,6 +144,14 @@ public enum FixedCode implements ReportCode {
     E_TNTN( "Table name not ADQL table_name" ),
     E_TOVO( "Multiple examples RDFa vocab attributes" ),
     E_TRSV( "Table name ADQL reserved word" ),
+    E_TSC0( "Missing TAP_SCHEMA column" ),
+    E_TSCT( "Wrong type for TAP_SCHEMA column" ),
+    E_TSAZ( "TAP_SCHEMA arraysize syntax error" ),
+    E_TSSZ( "TAP_SCHEMA size/arraysize inconsistency" ),
+    E_TST0( "Missing TAP_SCHEMA table" ),
+    E_TSTD( "TAP_SCHEMA column not flagged Std" ),
+    E_TSLN( "Missing TAP 1.1 TAP_SCHEMA foreign key" ),
+    E_TSNL( "Null values in non-nullable column" ),
     E_TTOO( "Multiple result TABLEs" ),
     E_TYPX( "ObsCore datatype incompatibilty" ),
     E_UDFE( "Bad UDF declaration" ),
@@ -159,6 +175,12 @@ public enum FixedCode implements ReportCode {
     W_CEZZ( "Compression" ),
     W_CIDX( "Index flag mismatch" ),
     W_CLUN( "Columns content" ),
+    W_CPAN( "Deprecated declaration of anonymous resource" ),
+    W_CPI2( "No unique std interface in capability" ),
+    W_CPS2( "Duplicated security method ID" ),
+    W_CPSM( "Unknown security method ID" ),
+    W_CPUL( "Non-standard access URL for standard interface" ),
+    W_CPUR( "Service and declared TAP base URL mismatch" ),
     W_CTYP( "Datatype possible mismatch" ),
     W_CUCD( "UCD mismatch" ),
     W_CULF( "Custom language feature type" ),
@@ -167,7 +189,6 @@ public enum FixedCode implements ReportCode {
     W_DMDC( "DataModel not declared" ),
     W_DMSS( "DataModel subset undeclared" ),
     W_DQU2( "Unknown value for post-table QUERY_STATUS" ),
-    W_DSUC( "Non-error return from duff query" ),
     W_EX00( "No examples in examples document" ),
     W_EXUL( "Undeclared query language" ),
     W_EXVC( "Potentially problematic examples RDFa vocab attribute" ),
@@ -188,10 +209,13 @@ public enum FixedCode implements ReportCode {
     W_QERR( "Query failed" ),
     W_QTYP( "Query/result column type possible mismatch" ),
     W_RDIO( "Resource read error" ),
+    W_SPPA( "Spurious query parameter changes result" ),
+    W_TADH( "Tables endpoint declaration/presence mismatch" ),
     W_TBNF( "Tables endpoint missing" ),
     W_TBUN( "Tables content" ),
     W_TFMT( "Non-ISO-8601 result" ),
     W_TSDL( "Bad time format in table data" ),
+    W_TSZ1( "Single-element array declared" ),
     W_TYPI( "ObsCore datatype mismatch" ),
     W_UNPH( "UNKNOWN phase" ),
     W_UNSC( "Foreign schema used in validation" ),
@@ -201,6 +225,7 @@ public enum FixedCode implements ReportCode {
     W_ZRES( "Resolver not used?" ),
 
     F_CAIO( "Bad capabilities service URL" ),
+    F_CAP0( "No capabilities available" ),
     F_CAPC( "XML parser error" ),
     F_DTIO( "Unexpected table read error" ),
     F_EXNO( "No examples document" ),
@@ -237,7 +262,9 @@ public enum FixedCode implements ReportCode {
     I_QJOB( "Submitted query URL" ),
     I_QSUB( "Submitted query" ),
     I_SCHM( "Use TAP_SCHEMA for metadata" ),
+    I_TAPV( "TAP validation version" ),
     I_TMAX( "Table test count" ),
+    I_TSNS( "Non-standard TAP_SCHEMA columns" ),
     I_TURL( "Reading table metadata" ),
     I_VURL( "Validation" ),
     I_VUWS( "UWS job document version" ),
