@@ -8,15 +8,14 @@ import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import uk.ac.starlink.table.RowSequence;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.Tables;
 import uk.ac.starlink.vo.ColumnMeta;
-import uk.ac.starlink.vo.EndpointSet;
 import uk.ac.starlink.vo.SchemaMeta;
 import uk.ac.starlink.vo.TableMeta;
 import uk.ac.starlink.vo.TapCapability;
 import uk.ac.starlink.vo.TapQuery;
+import uk.ac.starlink.vo.TapService;
 
 /**
  * Validation stage for testing ObsCore data model metadata and content.
@@ -56,7 +55,7 @@ public class ObsTapStage implements Stage {
         return "Test implementation of ObsCore Data Model";
     }
 
-    public void run( Reporter reporter, EndpointSet endpointSet ) {
+    public void run( Reporter reporter, TapService tapService ) {
 
         /* Check prerequisites. */
         boolean obsDeclared;
@@ -127,7 +126,7 @@ public class ObsTapStage implements Stage {
                        + ( is11 ? "1.1" : "1.0" ) );
 
         /* Run tests. */
-        new ObsTapRunner( reporter, endpointSet, obsMeta, is11, tapRunner_ )
+        new ObsTapRunner( reporter, tapService, obsMeta, is11, tapRunner_ )
            .run();
     }
 
@@ -135,12 +134,6 @@ public class ObsTapStage implements Stage {
      * Determines whether a table capability reports conformance to the
      * ObsCore data model.  If not, an appropriate report is made.
      *
-     * <p>If both are declared, only return v1.1.
-     * According to PR-ObsCore-v1.1-20160330, ObsCore 1.1 is supposed
-     * to be a superset of ObsCore 1.0, although in fact it is not.
-     * This code needs revisiting when that has been sorted out in
-     * the standard document.
-     *
      * @param  reporter   reporter
      * @param  tcap    tap capability object
      * @return   version of ObsCore model indicated, or null if no ObsCore
@@ -244,7 +237,7 @@ public class ObsTapStage implements Stage {
      */
     private static class ObsTapRunner implements Runnable {
         private final Reporter reporter_;
-        private final EndpointSet endpointSet_;
+        private final TapService tapService_;
         private final TapRunner tRunner_;
         private final Map<String,ColumnMeta> gotColMap_;
         private final Map<String,ObsCol> reqColMap_;
@@ -254,15 +247,15 @@ public class ObsTapStage implements Stage {
          * Constructor.
          *
          * @param  reporter  validation message destination
-         * @param  endpointSet  locations of TAP services
+         * @param  tapService  TAP service description
          * @param  obsMeta   table metadata for ivoa.ObsCore table
          * @param  is11   true for ObsCore-1.1, false for ObsCore-1.0
          * @param  tapRunner  runs TAP queries
          */
-        ObsTapRunner( Reporter reporter, EndpointSet endpointSet,
+        ObsTapRunner( Reporter reporter, TapService tapService,
                       TableMeta obsMeta, boolean is11, TapRunner tapRunner ) {
             reporter_ = reporter;
-            endpointSet_ = endpointSet;
+            tapService_ = tapService;
             gotColMap_ = toMap( obsMeta.getColumns() );
             tRunner_ = tapRunner;
             reqColMap_ = createMandatoryColumns( is11 );
@@ -346,11 +339,11 @@ public class ObsTapStage implements Stage {
         private void checkMetadata( ColumnMeta gotCol, ObsCol stdCol ) {
             String cname = gotCol.getName();
             compareItem( cname, "Utype", FixedCode.E_CUTP,
-                         stdCol.utype_, gotCol.getUtype(), false );
+                         stdCol.utype_, gotCol.getUtype(), false, null );
             compareItem( cname, "UCD", FixedCode.E_CUCD,
-                         stdCol.ucd_, gotCol.getUcd(), false );
+                         stdCol.ucd_, gotCol.getUcd(), false, stdCol.ucdOnce_ );
             compareItem( cname, "Unit", FixedCode.E_CUNI,
-                         stdCol.unit_, gotCol.getUnit(), true );
+                         stdCol.unit_, gotCol.getUnit(), true, null );
             checkType( gotCol, stdCol );
         }
 
@@ -447,7 +440,7 @@ public class ObsTapStage implements Stage {
          * @param   cname  column name
          * @param   [low,high] inclusive range permitted
          */
-        private void checkRange( String cname, Comparable[] range ) {
+        private void checkRange( String cname, Comparable<?>[] range ) {
             String adql = new StringBuffer()
                .append( "SELECT TOP 1 " )
                .append( cname )
@@ -564,14 +557,17 @@ public class ObsTapStage implements Stage {
          * @param  obsValue  correct value for metadata item
          * @param  gotValue  actual value of metadata item
          * @param  isCaseSensitive  true iff value comparison is case-sensitive
+         * @param  obsValueOnce   if not null, indicates a value that used
+         *                        to be OK, but has been changed by an
+         *                        ObsCore Erratum
          */
         private void compareItem( String colName, String itemName,
                                   ReportCode code,
                                   String obsValue, String gotValue,
-                                  boolean isCaseSensitive ) {
-            String badUcd = "meta.ref.uri;meta.curation";
-            String vGot = String.valueOf( gotValue );
-            String vObs = String.valueOf( obsValue );
+                                  boolean isCaseSensitive,
+                                  String obsValueOnce ) {
+            String vGot = gotValue == null ? "null" : gotValue;
+            String vObs = obsValue == null ? "null" : obsValue;
             if ( isCaseSensitive ? ( ! vGot.equals( vObs ) )
                                  : ( ! vGot.equalsIgnoreCase( vObs ) ) ) {
                 StringBuffer sbuf = new StringBuffer()
@@ -583,11 +579,11 @@ public class ObsTapStage implements Stage {
                     .append( gotValue )
                     .append( " != " )
                     .append( obsValue );
-                if ( badUcd.equalsIgnoreCase( obsValue ) ) {
-                    sbuf.append( "; NOTE \"" )
-                        .append( badUcd )
-                        .append( "\" is bad UCD1+ syntax" )
-                        .append( " - ObsCore/UCD erratum required" );
+                if ( obsValueOnce != null &&
+                     isCaseSensitive ? vGot.equals( obsValueOnce)
+                                     : vGot.equalsIgnoreCase( obsValueOnce ) ) {
+                    sbuf.append( " (used to be correct," )
+                        .append( " but changed by ObsCore Erratum)" );
                 }
                 reporter_.report( code, sbuf.toString() );
             }
@@ -602,36 +598,9 @@ public class ObsTapStage implements Stage {
          * @return   table result, or null
          */
         private TableData runQuery( String adql ) {
-            TapQuery tq = new TapQuery( endpointSet_, adql, null );
+            TapQuery tq = new TapQuery( tapService_, adql, null );
             StarTable table = tRunner_.getResultTable( reporter_, tq );
-            if ( table == null ) {
-                return null;
-            }
-            final List<Object[]> rowList = new ArrayList<Object[]>();
-            try {
-                RowSequence rseq = table.getRowSequence();
-                try {
-                    while ( rseq.next() ) {
-                        rowList.add( rseq.getRow() );
-                    }
-                }
-                finally {
-                    rseq.close();
-                }
-            }
-            catch ( IOException e ) {
-                reporter_.report( FixedCode.F_TIOF,
-                                  "Error reading result table", e );
-                return null;
-            }
-            return new TableData() {
-                public int getRowCount() {
-                    return rowList.size();
-                }
-                public Object getCell( int irow, int icol ) {
-                    return rowList.get( irow )[ icol ];
-                }
-            };
+            return TableData.createTableData( reporter_, table );
         }
     }
 
@@ -657,7 +626,7 @@ public class ObsTapStage implements Stage {
             new ObsCol( "obs_id", Type.VARCHAR,
                         "DataID.observationID", "meta.id" ),
             new ObsCol( "obs_publisher_did", Type.VARCHAR,
-                        "Curation.PublisherDID", "meta.ref.uri;meta.curation" ),
+                        "Curation.PublisherDID", "meta.ref.ivoid" ),
             new ObsCol( "access_url", Type.CLOB,
                         "Access.Reference", "meta.ref.url" ),
             new ObsCol( "access_format", Type.VARCHAR,
@@ -722,6 +691,7 @@ public class ObsTapStage implements Stage {
                         "Char.ObservableAxis.ucd", "meta.ucd" ),
 
             // Note ObsCore 1.1 inconsistent on mandatoryness of pol_states.
+            // Clarified in ObsCore-1.1 Erratum #2: it is mandatory.
             new ObsCol( "pol_states", Type.VARCHAR,
                         "Char.PolarizationAxis.stateList",
                         "meta.code;phys.polarization" ),
@@ -777,6 +747,9 @@ public class ObsTapStage implements Stage {
         map.get( "obs_id" ).nullForbidden_ = true;
         map.get( "obs_publisher_did" ).nullForbidden_ = true;
 
+        /* ObsCore 1.1 Erratum #1. */
+        map.get( "obs_publisher_did" ).ucdOnce_ = "meta.ref.uri;meta.curation";
+
         return map;
     }
 
@@ -803,7 +776,7 @@ public class ObsTapStage implements Stage {
             new ObsCol( "obs_title", Type.VARCHAR,
                         "DataID.Title", "meta.title;obs" ),
             new ObsCol( "publisher_id", Type.VARCHAR,
-                        "Curation.PublisherID", "meta.ref.uri;meta.curation" ),
+                        "Curation.PublisherID", "meta.ref.ivoid" ),
             new ObsCol( "bib_reference", Type.VARCHAR,
                         "Curation.Reference", "meta.bib.bibcode" ),
             new ObsCol( "data_rights", Type.VARCHAR,
@@ -870,7 +843,7 @@ public class ObsTapStage implements Stage {
                         "meta.code.qual" ),
             new ObsCol( "o_stat_error", Type.DOUBLE,
                         "Char.ObservableAxis.Accuracy.StatError.refval.value",
-                        "stat.error;phot.flux" ),
+                        "stat.error" ),
             new ObsCol( "proposal_id", Type.VARCHAR,
                         "Provenance.Proposal.identifier",
                         "meta.id;obs.proposal" ),
@@ -911,6 +884,10 @@ public class ObsTapStage implements Stage {
             "absolute", "relative", "normalized", "any",
         };
 
+        /* ObsCore 1.1 Erratum #1. */
+        map.get( "publisher_id" ).ucdOnce_ = "meta.ref.uri;meta.curation";
+        map.get( "o_stat_error" ).ucdOnce_ = "stat.error;phot.flux";
+
         return map;
     }
 
@@ -1022,28 +999,6 @@ public class ObsTapStage implements Stage {
         return name.toLowerCase();
     }
 
-    /**
-     * Contains the data from a table in easy to digest form (no IOExceptions).
-     * Suitable for holding small tables.
-     */
-    private static abstract class TableData {
-
-        /**
-         * Returns number of rows.
-         *
-         * @return  row count
-         */
-        abstract int getRowCount();
-
-        /**
-         * Returns the value of a cell.
-         *
-         * @param  irow  row index
-         * @param  icol  column index
-         */
-        abstract Object getCell( int irow, int icol );
-    }
-
     /**
      * Enumeration for known versions of the ObsCore data model.
      */
@@ -1079,10 +1034,11 @@ public class ObsTapStage implements Stage {
         final String utype_;
         final String ucd_;
         final String unit_;
+        String ucdOnce_;
         boolean nullForbidden_;
         String[] hardOptions_;
         String[] softOptions_;
-        Comparable[] range_;   // [low, high] inclusive
+        Comparable<?>[] range_;   // [low, high] inclusive
 
         /**
          * Constructor including units.
