@@ -4,6 +4,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -17,12 +18,12 @@ import uk.ac.starlink.table.StoragePolicy;
 import uk.ac.starlink.util.DOMUtils;
 import uk.ac.starlink.vo.AdqlSyntax;
 import uk.ac.starlink.vo.ColumnMeta;
-import uk.ac.starlink.vo.EndpointSet;
 import uk.ac.starlink.vo.SchemaMeta;
 import uk.ac.starlink.vo.TableMeta;
 import uk.ac.starlink.vo.TapCapability;
 import uk.ac.starlink.vo.TapLanguage;
 import uk.ac.starlink.vo.TapQuery;
+import uk.ac.starlink.vo.TapService;
 import uk.ac.starlink.votable.VODocument;
 import uk.ac.starlink.votable.VOElement;
 import uk.ac.starlink.votable.VOStarTable;
@@ -60,7 +61,9 @@ public class QueryStage implements Stage {
              + " mode";
     }
 
-    public void run( Reporter reporter, EndpointSet endpointSet ) {
+    public void run( Reporter reporter, TapService tapService ) {
+
+        /* Acquire sufficient metadata to be able to write test queries. */
         SchemaMeta[] smetas = metaHolder_.getTableMetadata();
         String[] adqlLangs = capHolder_ == null
                            ? null
@@ -96,8 +99,23 @@ public class QueryStage implements Stage {
         TableMeta[] tmetas =
             ( dataTables.size() > 0 ? dataTables : allTables )
            .toArray( new TableMeta[ 0 ] );
-        new Querier( reporter, endpointSet, tmetas, adqlLangs ).run();
-        runDuffQuery( reporter, endpointSet );
+
+        /* Run some test queries that ought to succeed. */
+        new Querier( reporter, tapService, tmetas, adqlLangs ).run();
+
+        /* Run some test queries that ought to fail. */
+        Map<String,String> extraParams = new HashMap<String,String>();
+        runErrorQuery( reporter,
+                       new TapQuery( tapService, "DUFF QUERY", extraParams ),
+                       "bad ADQL" );
+        extraParams.put( "LANG", "OOBLECK" );
+        runErrorQuery( reporter,
+                       new TapQuery( tapService,
+                                     "SELECT TOP 1 * FROM TAP_SCHEMA.tables",
+                                     extraParams ),
+                       "unknown query language" );
+
+        /* Summarise. */
         tapRunner_.reportSummary( reporter );
     }
 
@@ -132,36 +150,40 @@ public class QueryStage implements Stage {
     }
 
     /**
-     * Runs a query with an intentionally nonsensical query string,
-     * performing checks on whether the service issues error reports
-     * in the correct way.
+     * Runs a query that ought to fail.  If it apparently succeeds,
+     * this method reports an error.  Reports are also made if the
+     * service does not issue flag the failure correctly.
      *
      * @param  reporter  validation message destination
-     * @param  endpointSet  locations of TAP services
+     * @param  tq   query expected to fail
+     * @param  errorType  short description of what's wrong with the query,
+     *                    to be used in reporting messages
      */
-    private void runDuffQuery( Reporter reporter, EndpointSet endpointSet ) {
-        String duffAdql = "DUFF QUERY";
+    private void runErrorQuery( Reporter reporter, TapQuery tq,
+                                String errorType ) {
+        String pErrorType = " (" + errorType + ")";
         reporter.report( FixedCode.I_DUFF,
-                         "Submitting duff query: " + duffAdql );
+                         "Submitting query expected to fail" + pErrorType
+                       + ": " + tq.getAdql() );
         VOElement resultsEl;
         try {
-            TapQuery tq = new TapQuery( endpointSet, duffAdql, null );
             InputStream in = tapRunner_.readResultInputStream( reporter, tq );
             VODocument doc = tapRunner_.readResultDocument( reporter, in );
             resultsEl = tapRunner_.getResultsResourceElement( reporter, doc );
         }
         catch ( SAXException e ) {
             reporter.report( FixedCode.E_DFSF,
-                             "TAP result parse failed for \""
-                           + duffAdql + "\"", e );
+                             "TAP result parse failed for bad query"
+                           + pErrorType, e );
             return;
         }
         catch ( IOException e ) {
             reporter.report( FixedCode.E_DFIO,
-                             "TAP job failed for duff query", e );
+                             "TAP job failed for bad query" + pErrorType, e );
             return;
         }
         VOElement statusInfo = null;
+        boolean hasTable = false;
         for ( Node node = resultsEl.getFirstChild(); node != null;
               node = node.getNextSibling() ) {
             if ( node instanceof VOElement ) {
@@ -170,8 +192,9 @@ public class QueryStage implements Stage {
                 boolean isStatusInfo =
                     "INFO".equals( name ) &&
                     "QUERY_STATUS".equals( el.getAttribute( "name" ) );
-                boolean isTable =
-                    "TABLE".equals( name );
+                if ( "TABLE".equals( name ) ) {
+                    hasTable = true;
+                }
                 if ( isStatusInfo ) {
                     if ( statusInfo != null ) {
                         reporter.report( FixedCode.E_EST1,
@@ -180,23 +203,22 @@ public class QueryStage implements Stage {
                     }
                     statusInfo = el;
                 }
-                if ( isTable ) {
-                    reporter.report( FixedCode.W_HSTB,
-                                     "Return from duff query contains TABLE" );
-                }
             }
         }
         if ( statusInfo == null ) {
             reporter.report( FixedCode.E_DNST,
                              "Missing <INFO name='QUERY_STATUS'> element "
-                           + "for duff query" );
+                           + "for bad query" + pErrorType );
         }
         else {
             String status = statusInfo.getAttribute( "value" );
-            if ( "ERROR".equals( status ) ) {
 
-                /* Error (presumably parse error) reported as expected
-                 * for duff query. */
+            /* Error reported as expected for duff query - basically correct. */
+            if ( "ERROR".equals( status ) ) {
+                if ( hasTable ) {
+                    reporter.report( FixedCode.W_HSTB,
+                                     "ERROR response contains TABLE" );
+                }
                 String err = DOMUtils.getTextContent( statusInfo );
                 if ( err == null || err.trim().length() == 0 ) {
                     reporter.report( FixedCode.W_NOMS,
@@ -205,10 +227,19 @@ public class QueryStage implements Stage {
                                    + "element has no message content" );
                 }
             }
+
+            /* Apparent query success, report trouble. */
             else if ( "OK".equals( status ) ) {
-                reporter.report( FixedCode.W_DSUC,
-                                 "Service reports OK from duff query" );
+                String msg = new StringBuffer()
+                    .append( hasTable ? "Apparent success"
+                                      : "Service reports OK (but no TABLE)" )
+                    .append( " from bad query" )
+                    .append( pErrorType )
+                    .toString();
+                reporter.report( FixedCode.E_DSUC, msg );
             }
+
+            /* Badly reported status. */
             else {
                 String msg = new StringBuffer()
                     .append( "QUERY_STATUS INFO has unknown value " )
@@ -225,7 +256,7 @@ public class QueryStage implements Stage {
      */
     private class Querier implements Runnable {
         private final Reporter reporter_;
-        private final EndpointSet endpointSet_;
+        private final TapService tapService_;
         private final String[] adqlLangs_;
         private final TableMeta tmeta1_;
         private final TableMeta tmeta2_;
@@ -235,18 +266,18 @@ public class QueryStage implements Stage {
          * Constructor.
          *
          * @param  reporter  validation message destination
-         * @param  endpointSet  locations of TAP services
+         * @param  tapService   TAP service description
          * @param  tmetas  metadata for known tables
          */
-        Querier( Reporter reporter, EndpointSet endpointSet, TableMeta[] tmetas,
+        Querier( Reporter reporter, TapService tapService, TableMeta[] tmetas,
                  String[] adqlLangs ) {
             reporter_ = reporter;
-            endpointSet_ = endpointSet;
+            tapService_ = tapService;
             adqlLangs_ = adqlLangs;
 
             /* Pick some representative tables for later testing.
              * Do it by choosing tables with column counts near the median. */
-            tmetas = (TableMeta[]) tmetas.clone();
+            tmetas = tmetas.clone();
             Arrays.sort( tmetas, new Comparator<TableMeta>() {
                 public int compare( TableMeta tm1, TableMeta tm2 ) {
                     return tm1.getColumns().length - tm2.getColumns().length;
@@ -317,6 +348,38 @@ public class QueryStage implements Stage {
                 }
             }
 
+            /* Check that a spurious request parameter is ignored
+             * (TAP 1.0 sec 2.3, TAP 1.1 PR sec 2.7). */
+            if ( t1 != null ) {
+                Map<String,String> extraParams = new HashMap<String,String>();
+                String spuriousName = "DUMMY";
+                String spuriousValue = "ignore-me";
+                extraParams.put( spuriousName, spuriousValue );
+                TapQuery tq = new TapQuery( tapService_, tAdql, extraParams );
+                StarTable t1a = tapRunner_.getResultTable( reporter_, tq );
+                if ( t1a == null ) {
+                    String msg = new StringBuffer()
+                       .append( "Adding spurious request parameter " )
+                       .append( spuriousName )
+                       .append( "=" )
+                       .append( spuriousValue )
+                       .append( " breaks query" )
+                       .toString();
+                    reporter_.report( FixedCode.E_SPPA, msg );
+                }
+                else if ( t1a.getRowCount() != t1.getRowCount() ||
+                          t1a.getColumnCount() != t1.getColumnCount() ) {
+                    String msg = new StringBuffer()
+                       .append( "Adding spurious request parameter " )
+                       .append( spuriousName )
+                       .append( "=" )
+                       .append( spuriousValue )
+                       .append( " changes query result" )
+                       .toString();
+                    reporter_.report( FixedCode.W_SPPA, msg );
+                }
+            }
+
             /* Check that all known variants (typically "ADQL" and "ADQL-2.0")
              * work the same. */
             if ( adqlLangs_ != null && adqlLangs_.length > 1 ) {
@@ -334,7 +397,7 @@ public class QueryStage implements Stage {
                         new HashMap<String,String>();
                     extraParams.put( "LANG", lang );
                     TapQuery tq =
-                        new TapQuery( endpointSet_, vAdql, extraParams );
+                        new TapQuery( tapService_, vAdql, extraParams );
                     StarTable result =
                         tapRunner_.getResultTable( reporter_, tq );
                     ( result == null ? failLangList
@@ -448,7 +511,7 @@ public class QueryStage implements Stage {
             if ( maxrec >= 0 ) {
                 extraParams.put( "MAXREC", Integer.toString( maxrec ) );
             }
-            TapQuery tq = new TapQuery( endpointSet_, adql, extraParams );
+            TapQuery tq = new TapQuery( tapService_, adql, extraParams );
             StarTable table = tapRunner_.getResultTable( reporter_, tq );
             if ( table != null ) {
                 int nrow = (int) table.getRowCount();
