@@ -5,6 +5,7 @@ import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.Tables;
 import uk.ac.starlink.task.ChoiceParameter;
 import uk.ac.starlink.task.Environment;
+import uk.ac.starlink.task.ExecutionException;
 import uk.ac.starlink.task.Parameter;
 import uk.ac.starlink.task.ParameterValueException;
 import uk.ac.starlink.task.StringParameter;
@@ -217,8 +218,8 @@ public class PixSample extends MapperTask {
             outsysParam_.setNullPermitted( true );
         }
 
-        public Parameter[] getParameters() {
-            return new Parameter[] {
+        public Parameter<?>[] getParameters() {
+            return new Parameter<?>[] {
                 schemeParam_,
                 modeParam_,
                 lonParam_,
@@ -258,7 +259,19 @@ public class PixSample extends MapperTask {
                     }
                     else {
                         assert scheme == HealpixScheme.AUTO;
-                        isNested = PixSampler.inferNested( pixTable );
+                        Boolean guessNest = PixSampler.inferNested( pixTable );
+                        if ( guessNest == null ) {
+                            String msg = new StringBuffer()
+                                .append( "Can't determine ordering scheme" )
+                                .append( "; please supply " )
+                                .append( schemeParam_.getName() )
+                                .append( " parameter" )
+                                .toString();
+                            throw new ExecutionException( msg );
+                        }
+                        else {
+                            isNested = guessNest.booleanValue();
+                        }
                     }
                     PixSampler pixSampler =
                         new PixSampler( pixTable, nside, isNested );
@@ -320,15 +333,14 @@ public class PixSample extends MapperTask {
                 "This is usually, but not necessarily, FITS.",
                 "</p>",
             } );
-            pixFmtParam.setStringDefault( "fits" );
 
             pixFilterParam_ = new FilterParameter( "pcmd" );
             pixFilterParam_.setTableDescription( "pixel data table", 
                                                  pixTableParam_, Boolean.TRUE );
         }
 
-        public Parameter[] getParameters() {
-            return new Parameter[] {
+        public Parameter<?>[] getParameters() {
+            return new Parameter<?>[] {
                 inTableParam_,
                 inTableParam_.getFormatParameter(),
                 inFilterParam_,
