@@ -20,6 +20,7 @@ public abstract class VersionDetail {
     private static final VersionDetail V11;
     private static final VersionDetail V12;
     private static final VersionDetail V13;
+    private static final VersionDetail V14;
     private static final VersionDetail DUMMY = new DummyVersionDetail();
     private static final Map<VOTableVersion,VersionDetail> VERSION_MAP =
             createMap( new VersionDetail[] {
@@ -27,6 +28,7 @@ public abstract class VersionDetail {
         V11 = new VersionDetail11( VOTableVersion.V11 ),
         V12 = new VersionDetail12( VOTableVersion.V12 ),
         V13 = new VersionDetail13( VOTableVersion.V13 ),
+        V14 = new VersionDetail14( VOTableVersion.V14 ),
     } );
 
     /**
@@ -214,7 +216,8 @@ public abstract class VersionDetail {
                 hasID = true;
                 hasName = true;
                 map.put( "ref",
-                         new RefChecker( new String[] { "COOSYS", "GROUP" } ) );
+                         new RefChecker( new String[] { "COOSYS", "TIMESYS",
+                                                        "GROUP" } ) );
             }
             else if ( "FITS".equals( name ) ) {
             }
@@ -241,8 +244,8 @@ public abstract class VersionDetail {
                 hasName = true;
                 map.put( "value", new ParamHandler.ValueChecker() );
                 map.put( "ref",
-                         new RefChecker( new String[] { "COOSYS", "GROUP",
-                                                        "FIELD", } ) );
+                         new RefChecker( new String[] { "COOSYS", "TIMESYS",
+                                                        "FIELD", "GROUP" } ) );
             }
             else if ( "RESOURCE".equals( name ) ) {
                 hasID = true;
@@ -261,6 +264,11 @@ public abstract class VersionDetail {
             else if ( "TD".equals( name ) ) {
                 map.put( "ref", new RefChecker( new String[ 0 ] ) );
             }
+            else if ( "TIMESYS".equals( name ) ) {
+                hasID = true;
+                map.put( "timescale", VocabChecker.TIMESCALE );
+                map.put( "refposition", VocabChecker.REFPOSITION );
+            }
             else if ( "TR".equals( name ) ) {
             }
             else if ( "VALUES".equals( name ) ) {
@@ -389,7 +397,32 @@ public abstract class VersionDetail {
 
         protected Map<String,AttributeChecker>
                createAttributeCheckers( String name ) {
-            return V12.createAttributeCheckers( name );
+            Map<String,AttributeChecker> map =
+                V12.createAttributeCheckers( name );
+            if ( "FIELD".equals( name ) ||
+                 "PARAM".equals( name ) ) {
+                map.put( "arraysize", new ArraysizeChecker() );
+            }
+            return map;
+        }
+    }
+
+    /**
+     * VersionDetail implementation for VOTable 1.4.
+     */
+    private static class VersionDetail14 extends VersionDetail {
+
+        VersionDetail14( VOTableVersion version ) {
+            super( version );
+        }
+
+        protected ElementHandler createElementHandler( String name ) {
+            return V13.createElementHandler( name );
+        }
+
+        protected Map<String,AttributeChecker>
+                createAttributeCheckers( String name ) {
+            return V13.createAttributeCheckers( name );
         }
     }
 
