@@ -398,7 +398,7 @@ public class FuncTest extends TestCase {
         }
         catch ( NumberFormatException e ) {
         }
-        
+
         assertEquals( 2.75f, Conversions.toFloat( 2.75 ) );
 
         assertEquals( -Math.PI, Conversions.toDouble( -Math.PI ) );
@@ -415,6 +415,18 @@ public class FuncTest extends TestCase {
         assertEquals( 42, Conversions.fromHex( "2A" ) );
         assertEquals( 42, Conversions.fromHex( " 2a" ) );
         assertEquals( 42, Conversions.fromHex( "2a " ) );
+
+        assertEquals( 101, Conversions.parseBigDecimal( "101" ).doubleValue() );
+        assertEquals( 101, Conversions.parseBigInteger( "101" )
+                                      .intValueExact() );
+        assertEquals( -2e19,
+                      Conversions
+                     .parseBigInteger("-20000000000000000023").doubleValue() );
+        assertTrue( Conversions.parseBigInteger("18446744073709551616")
+                               .testBit(64) );
+
+        assertEquals( -1, Conversions.parseBigDecimal("101")
+                         .compareTo(Conversions.parseBigDecimal("102")) );
     }
 
     public void testDistances() {
@@ -619,8 +631,8 @@ public class FuncTest extends TestCase {
         assertEquals( "some text", Strings.trim( "  some text  " ) );
         assertEquals( "some text", Strings.trim( "some text" ) );
 
-        assertEquals( "RR%20Lyr", Strings.urlEncode( "RR Lyr" ) );
-        assertEquals( "RR Lyr", Strings.urlDecode( "RR%20Lyr" ) );
+        assertEquals( "RR%20Lyr", URLs.urlEncode( "RR Lyr" ) );
+        assertEquals( "RR Lyr", URLs.urlDecode( "RR%20Lyr" ) );
 
         // These examples from sec 3.5.1 of IAU-endorsed document
         // "Specifications concerning designations" at
@@ -640,31 +652,46 @@ public class FuncTest extends TestCase {
         assertTrue( Double.isNaN( Strings.desigToDec( "TYC J190012230210" ) ) );
 
         assertEquals( "https://ui.adsabs.harvard.edu/abs/2018A%26A...616A...2L",
-                      Strings.bibcodeUrl( "2018A&A...616A...2L" ) );
-        assertNull( Strings.bibcodeUrl( null ) );
-        assertNull( Strings.bibcodeUrl( "Fredor" ) );
+                      URLs.bibcodeUrl( "2018A&A...616A...2L" ) );
+        assertNull( URLs.bibcodeUrl( null ) );
+        assertNull( URLs.bibcodeUrl( "Fredor" ) );
         assertEquals( "https://doi.org/10.3390/informatics4030018",
-                      Strings.doiUrl( "10.3390/informatics4030018" ) );
+                      URLs.doiUrl( "10.3390/informatics4030018" ) );
         assertEquals( "https://doi.org/10.3390/informatics4030018",
-                      Strings.doiUrl( "doi:10.3390/informatics4030018" ) );
-        assertNull( Strings.doiUrl( "Fredor" ) );
-        assertNull( Strings.doiUrl( null ) );
+                      URLs.doiUrl( "doi:10.3390/informatics4030018" ) );
+        assertNull( URLs.doiUrl( "Fredor" ) );
+        assertNull( URLs.doiUrl( null ) );
         assertEquals( "https://arxiv.org/abs/1804.09379",
-                      Strings.arxivUrl( "1804.09379" ) );
+                      URLs.arxivUrl( "1804.09379" ) );
         assertEquals( "https://arxiv.org/abs/1804.09379",
-                      Strings.arxivUrl( "arXiv:1804.09379" ) );
-        assertNull( Strings.arxivUrl( null ) );
-        assertNull( Strings.arxivUrl( "Fredor" ) );
+                      URLs.arxivUrl( "arXiv:1804.09379" ) );
+        assertEquals( "https://arxiv.org/abs/1110.0528",
+                      URLs.arxivUrl( "arxiv:1110.0528" ) );
+        assertNull( URLs.arxivUrl( null ) );
+        assertNull( URLs.arxivUrl( "Fredor" ) );
 
         assertEquals( "http://simbad.u-strasbg.fr/simbad/sim-id?Ident="
                     + "Beta%20Pictoris",
-                      Strings.simbadUrl( "Beta Pictoris" ) );
-        assertNull( Strings.simbadUrl( null ) );
-        assertNull( Strings.simbadUrl( " " ) );
+                      URLs.simbadUrl( "Beta Pictoris" ) );
+        assertNull( URLs.simbadUrl( null ) );
+        assertNull( URLs.simbadUrl( " " ) );
         assertEquals( "http://ned.ipac.caltech.edu/byname?objname=NGC%203952",
-                      Strings.nedUrl( "NGC 3952" ) );
-        assertNull( Strings.nedUrl( null ) );
-        assertNull( Strings.nedUrl( "    " ) );
+                      URLs.nedUrl( "NGC 3952" ) );
+        assertNull( URLs.nedUrl( null ) );
+        assertNull( URLs.nedUrl( "    " ) );
+
+        assertEquals( "http://alasky.u-strasbg.fr/hips-image-services/hips2fits"
+                    + "?hips=CDS/P/DSS2/color&format=png"
+                    + "&ra=56.75&dec=24.1125"
+                    + "&fov=1.50&width=300&height=300&projection=SIN",
+                      URLs.hips2fitsUrl( "CDS/P/DSS2/color", "png",
+                                         56.75, 24.1125, 1.5, 300 ) );
+    }
+
+    public void testURLs() {
+        assertEquals( "http://x.org/?a=1&b=two&c=3%264",
+                      URLs.paramsUrl( "http://x.org/",
+                                      "a", "1", "b", "two", "c", "3&4" ) );
     }
 
     private void checkDesig( String raSex, String decSex, String desig ) {
